////////////////////////////////////////////////////////////////////////////////
// File         : Baustein.h
// Subsystem    : Graph
// Component    : object networks
// Author       : Josef Huebl, (Josef.Huebl@sss.de)
// Description  : definition of the class Baustein
////////////////////////////////////////////////////////////////////////////////
// Copyright (C) Triple-S GmbH (www.sss.de) 2003
// Alle Rechte vorbehalten. All Rights Reserved.
//
// The reproduction, transmission or use of this document or its contents is
// not permitted without express written authority.
// Offenders will be liable for damages. All rights, including rights created
// by patent grant or registration of a utility model or design, are reserved.
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// History:
// 01.10.03    Josef Huebl       created
// 17.12.03    Josef Huebl       1. released version.
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Hints:      
//  
// Baustein is an important base class, which implements a corresponding handling of
// object (component or baustein) hierarchies and their relations in an H-net,
// whereby an object corresponds with a subcentre in an H-net.
//
// There exist elementary and non-elementary (composed) bausteins.
// when the hierarchy is broken down, the non-elementary bausteins are replaced
// by their corresponding H-net. 

// While elementary bausteins realise a specific functionality
// non-elementary (composed) bausteins are only used to structure a more complex model. 
//
// The main task of the class Baustein is the storing of pointers to all subbausteins
// of composed baustein and offering of connect-functions to model their inner network.  
//  
// For more information see the documents "S-net.doc" and "Messplatzverschaltung.doc".
//   
////////////////////////////////////////////////////////////////////////////////

#if !defined(AFX_BAUSTEIN_H__7A7BA91E_2DD1_41E7_8DA3_73318B2B9A1E__INCLUDED_)
#define AFX_BAUSTEIN_H__7A7BA91E_2DD1_41E7_8DA3_73318B2B9A1E__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <stdio.h>
#include <vector>
#include "HnetUniverse.h"

class Baustein  
{

public:
	Baustein( IsoTypeNode &class_isoType, int numberOfEntries, int numberOfExits, Baustein *owner, HnetUniverse *universe = NULL );
	virtual ~Baustein();

	// used by elemetary bausteins; implemented by composed bausteins
	virtual void MakeConnections( void ) {}; 

	int NumberOfEntries( void ) { return m_numberOfEntries; };
	int NumberOfExits( void ) { return m_numberOfExits; };
	int NumberOfSubbausteins( void ) { return m_numberOfSubbausteins; };

protected:
	HnetUniverse *m_universe;
	IsoTypeNode *m_classIsoTypePtr;
	IsoTypeNode m_isoType;
	bool m_elementary;
	Node m_subcentre;
	int m_numberOfEntries;
	int m_numberOfExits;
	NodeVector m_entry;
	NodeVector m_exit;
	int m_numberOfSubbausteins;
	std::vector< Baustein * >m_subbausteins;

public:
	Node Entry( int pos ) { return m_entry[ pos ]; };
	Node Exit( int pos ) { return m_exit[ pos ]; };
	// Attenion! An outer Entry is a inner Exit and vice versa!
	Node OuterEntry( int pos ) { return m_exit[ pos ]; };
	Node OuterExit( int pos ) { return m_entry[ pos ]; };

protected:
	void SetElementary( void ) { m_elementary = true; m_universe->GetHnet( m_isoType ).SetElementary(); };	
	void Connect( Node const &x, Node const &y ) { OwnHnet().Connect( x, y ); };
	void Connect( Baustein &baustein1, int exitPos, Baustein &baustein2, int entryPos );
	void ConnectParallel( Baustein &baustein1, int exitPos, Baustein &baustein2, int entryPos, int numberOfConnections = 1 );
	void ConnectAll( Baustein &baustein1, int exitPos, Baustein &baustein2, int entryPos, int numberOfExits = 1, int numberOfEntries = 1 );
	void BuiltClassIsoType( IsoTypeNode &class_isoType );

public:
	IsoTypeNode GetIsoType( void ) { return m_isoType; };
	void SetIsoType( const IsoTypeNode &isoType ) { m_isoType = isoType; };
	Node GetSubcentre( void ) { return m_subcentre; };
	void SetSubcentre( const Node &s );
	void MapSubcentresElementary( Hnet &hnet, const Node &s );
	Hnet &OwnHnet( void ) { return m_universe->GetHnet( m_isoType ); };
	void PrepareOuterEntriesExits( const Hnet &hnet ) { SetSubcentre( hnet.Root() ); };
	void PrepareOuterEntriesExits( const Snet &snet ) { SetSubcentre( snet.Root() ); };
	bool CheckConnect( Baustein &baustein1, int exitPos, Baustein &baustein2, int entryPos, int numberOfExits = 1, int numberOfEntries = 1 );

private:
	void AddSubbaustein( Baustein *subbaustein ) { m_subbausteins.push_back( subbaustein ); m_numberOfSubbausteins = m_subbausteins.size(); };
	IsoTypeNode CheckClassIsoType( IsoTypeNode &class_isoType );
};

#endif // !defined(AFX_BAUSTEIN_H__7A7BA91E_2DD1_41E7_8DA3_73318B2B9A1E__INCLUDED_)
